'''
Function:
	一些工具函数
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import os
import json


'''检查文件夹是否存在'''
def checkDir(dirpath):
	if not os.path.exists(dirpath):
		os.mkdir(dirpath)
		return False
	return True


'''导入配置文件'''
def loadConfig(filepath='config.json'):
	f = open(filepath, 'r', encoding='utf-8')
	return json.load(f)


'''清楚可能出问题的字符'''
def filterBadCharacter(string):
	string = string.replace('<em>', '').replace('</em>', '') \
				   .replace('<', '').replace('>', '').replace('\\', '').replace('/', '') \
				   .replace('?', '').replace(':', '').replace('"', '').replace('：', '') \
				   .replace('|', '').replace('？', '').replace('*', '')
	return string.strip().encode('utf-8', 'ignore').decode('utf-8')


'''秒转时分秒'''
def seconds2hms(seconds):
	m, s = divmod(seconds, 60)
	h, m = divmod(m, 60)
	return '%02d:%02d:%02d' % (h, m, s)


'''
Function:
	音乐下载器
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import sys
if __name__ == '__main__': from modules import *
else: from .modules import *


'''basic info'''
BASICINFO = '''************************************************************
Function: 音乐下载器 V2.1.6
Author: Charles
微信公众号: Charles的皮卡丘
操作帮助:
	输入r: 重新初始化程序(即返回主菜单)
	输入q: 退出程序
	下载多首歌曲: 选择想要下载的歌曲时,输入{1,2,5}可同时下载第1,2,5首歌
歌曲保存路径:
	当前路径下的%s文件夹内
************************************************************'''


'''音乐下载器'''
class musicdl():
	def __init__(self, configpath=None, config=None, **kwargs):
		self.config = loadConfig('config.json') if config is None else config
		self.logger_handle = Logger(self.config['logfilepath'])
		self.initializeAllSources()
	'''非开发人员外部调用'''
	def run(self, target_srcs=None):
		while True:
			print(BASICINFO % self.config.get('savedir'))
			# 音乐搜索
			user_input = self.dealInput('请输入歌曲搜索的关键词: ')
			target_srcs = ['baiduFlac', 'kugou', 'kuwo', 'qq', 'qianqian', 'netease', 'migu', 'xiami', 'joox'] if target_srcs is None else target_srcs
			search_results = self.search(user_input, target_srcs)
			# 打印搜索结果
			title = ['序号', '歌手', '歌名', '大小', '时长', '专辑', '来源']
			items = []
			records = {}
			idx = 0
			for key, values in search_results.items():
				for value in values:
					items.append([str(idx), value['singers'], value['songname'], value['filesize'], value['duration'], value['album'], value['source']])
					records.update({str(idx): value})
					idx += 1
			printTable(title, items)
			# 音乐下载
			user_input = self.dealInput('请输入想要下载的音乐编号: ')
			need_download_numbers = user_input.split(',')
			songinfos = []
			for item in need_download_numbers:
				songinfo = records.get(item, '')
				if songinfo: songinfos.append(songinfo)
			self.download(songinfos)
	'''音乐搜索'''
	def search(self, keyword, target_srcs):
		search_results = {}
		if 'baiduFlac' in target_srcs:
			try:
				search_results.update({'baiduFlac': self.baiduFlac.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('baiduFlac', keyword))
		if 'kugou' in target_srcs:
			try:
				search_results.update({'kugou': self.kugou.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('kugou', keyword))
		if 'kuwo' in target_srcs:
			try:
				search_results.update({'kuwo': self.kuwo.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('kuwo', keyword))
		if 'netease' in target_srcs:
			try:
				search_results.update({'netease': self.netease.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('netease', keyword))
		if 'qianqian' in target_srcs:
			try:
				search_results.update({'qianqian': self.qianqian.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('qianqian', keyword))
		if 'qq' in target_srcs:
			try:
				search_results.update({'qq': self.qq.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('qq', keyword))
		if 'migu' in target_srcs:
			try:
				search_results.update({'migu': self.migu.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('migu', keyword))
		if 'xiami' in target_srcs:
			try:
				search_results.update({'xiami': self.xiami.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('xiami', keyword))
		if 'joox' in target_srcs:
			try:
				search_results.update({'joox': self.joox.search(keyword)})
			except:
				self.logger_handle.warning('无法在%s中搜索 ——> %s...' % ('joox', keyword))
		return search_results
	'''音乐下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			if songinfo['source'] == 'baiduFlac':
				self.baiduFlac.download([songinfo])
			elif songinfo['source'] == 'kugou':
				self.kugou.download([songinfo])
			elif songinfo['source'] == 'kuwo':
				self.kuwo.download([songinfo])
			elif songinfo['source'] == 'netease':
				self.netease.download([songinfo])
			elif songinfo['source'] == 'qianqian':
				self.qianqian.download([songinfo])
			elif songinfo['source'] == 'qq':
				self.qq.download([songinfo])
			elif songinfo['source'] == 'migu':
				self.migu.download([songinfo])
			elif songinfo['source'] == 'xiami':
				self.xiami.download([songinfo])
			elif songinfo['source'] == 'joox':
				self.joox.download([songinfo])
	'''初始化所有支持的搜索/下载源'''
	def initializeAllSources(self):
		self.baiduFlac = baiduFlac(self.config, self.logger_handle)
		self.kugou = kugou(self.config, self.logger_handle)
		self.kuwo = kuwo(self.config, self.logger_handle)
		self.netease = netease(self.config, self.logger_handle)
		self.qianqian = qianqian(self.config, self.logger_handle)
		self.qq = qq(self.config, self.logger_handle)
		self.migu = migu(self.config, self.logger_handle)
		self.xiami = xiami(self.config, self.logger_handle)
		self.joox = joox(self.config, self.logger_handle)
	'''处理用户输入'''
	def dealInput(self, tip=''):
		user_input = input(tip)
		if user_input.lower() == 'q':
			self.logger_handle.info('ByeBye...')
			sys.exit()
		elif user_input.lower() == 'r':
			self.initializeAllSources()
			self.run()
		else:
			return user_input


'''run'''
if __name__ == '__main__':
	dl_client = musicdl('config.json')
	dl_client.run()


'''import all'''
from .qq import qq
from .kuwo import kuwo
from .migu import migu
from .joox import joox
from .xiami import xiami
from .kugou import kugou
from .netease import netease
from .qianqian import qianqian
from .baiduFlac import baiduFlac


'''
Function:
	qq音乐下载: https://y.qq.com/
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import random
import requests
from ..utils.misc import *
from ..utils.downloader import Downloader


'''QQ音乐下载类'''
class qq():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'qq'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		params = {
					'w': keyword,
					'format': 'json',
					'p': '1',
					'n': cfg['search_size_per_source']
				}
		response = self.session.get(self.search_url, headers=self.headers, params=params)
		all_items = response.json()['data']['song']['list']
		songinfos = []
		for item in all_items:
			params = {
						'guid': str(random.randrange(1000000000, 10000000000)),
						'loginUin': '3051522991',
						'format': 'json',
						'platform': 'yqq',
						'cid': '205361747',
						'uin': '3051522991',
						'songmid': item['songmid'],
						'needNewCode': '0'
					}
			ext = ''
			download_url = ''
			filesize = '-MB'
			for quality in [("A000", "ape", 800), ("F000", "flac", 800), ("M800", "mp3", 320), ("C400", "m4a", 128), ("M500", "mp3", 128)]:
				params['filename'] = '%s%s.%s' % (quality[0], item['songmid'], quality[1])
				response = self.session.get(self.mobile_fcg_url, headers=self.ios_headers, params=params)
				response_json = response.json()
				if response_json['code'] != 0: continue
				vkey = response_json.get('data', {}).get('items', [{}])[0].get('vkey', '')
				if vkey:
					ext = quality[1]
					download_url = 'http://dl.stream.qqmusic.qq.com/{}?vkey={}&guid={}&uin=3051522991&fromtag=64'.format('%s%s.%s' % (quality[0], item['songmid'], quality[1]), vkey, params['guid'])
					if ext in ['ape', 'flac']:
						filesize = item['size%s' % ext]
					elif ext in ['mp3', 'm4a']:
						filesize = item['size%s' % quality[-1]]
					break
			if not download_url:
				params = {
							'data': json.dumps({
												"req": {"module": "CDN.SrfCdnDispatchServer", "method": "GetCdnDispatch", "param": {"guid": "3982823384", "calltype": 0, "userip": ""}},
												"req_0": {"module": "vkey.GetVkeyServer", "method": "CgiGetVkey", "param": {"guid": "3982823384", "songmid": [item['songmid']], "songtype": [0], "uin": "0", "loginflag": 1, "platform": "20"}},
												"comm": {"uin": 0, "format": "json", "ct": 24, "cv": 0}
											})
						}
				response = self.session.get(self.fcg_url, headers=self.ios_headers, params=params)
				response_json = response.json()
				if response_json['code'] == 0 and response_json['req']['code'] == 0 and response_json['req_0']['code'] == 0:
					ext = '.m4a'
					download_url = str(response_json["req"]["data"]["freeflowsip"][0]) + str(response_json["req_0"]["data"]["midurlinfo"][0]["purl"])
					filesize = item['size128']
			if (not download_url) or (filesize == '-MB') or (filesize == 0): continue
			filesize = str(round(filesize/1024/1024, 2)) + 'MB'
			duration = int(item.get('interval', 0))
			songinfo = {
						'source': self.source,
						'songid': str(item['songmid']),
						'singers': filterBadCharacter(','.join([s.get('name', '') for s in item.get('singer', [])])),
						'album': filterBadCharacter(item.get('albumname', '-')),
						'songname': filterBadCharacter(item.get('songname', '-')),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(item.get('songname', '-'))]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': seconds2hms(duration)
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''初始化'''
	def __initialize(self):
		self.ios_headers = {
							'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
							'Referer': 'http://y.qq.com'
						}
		self.headers = {
						'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
						'Referer': 'http://y.qq.com'
					}
		self.search_url = 'https://c.y.qq.com/soso/fcgi-bin/client_search_cp'
		self.mobile_fcg_url = 'https://c.y.qq.com/base/fcgi-bin/fcg_music_express_mobile3.fcg'
		self.fcg_url = 'https://u.y.qq.com/cgi-bin/musicu.fcg'


'''
Function:
	JOOX音乐下载: https://www.joox.com/cn/login
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import json
import time
import base64
import requests
from ..utils.misc import *
from ..utils.downloader import Downloader


'''JOOX音乐下载类'''
class joox():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'joox'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		params = {
					'country': 'hk',
					'lang': 'zh_TW',
					'search_input': keyword,
					'sin': '0',
					'ein': cfg['search_size_per_source']
				}
		response = self.session.get(self.search_url, headers=self.headers, params=params)
		all_items = response.json()['itemlist']
		songinfos = []
		for item in all_items:
			params = {
						'songid': item['songid'],
						'lang': 'zh_cn',
						'country': 'hk',
						'from_type': '-1',
						'channel_id': '-1',
						'_': str(int(time.time()*1000))
					}
			response = self.session.get(self.songinfo_url, headers=self.headers, params=params)
			response_json = json.loads(response.text.replace('MusicInfoCallback(', '')[:-1])
			if response_json.get('code') != 0: continue
			for q_key in [('r320Url', '320'), ('r192Url', '192'), ('mp3Url', '128')]:
				download_url = response_json.get(q_key[0], '')
				if not download_url: continue
				filesize = str(round(int(json.loads(response_json['kbps_map'])[q_key[1]])/1024/1024, 2)) + 'MB'
				ext = 'mp3' if q_key[0] in ['r320Url', 'mp3Url'] else 'm4a'
			if not download_url: continue
			duration = int(item.get('playtime', 0))
			songinfo = {
						'source': self.source,
						'songid': str(item['songid']),
						'singers': filterBadCharacter(','.join([base64.b64decode(s['name']).decode('utf-8') for s in item.get('singer_list', [])])),
						'album': filterBadCharacter(response_json.get('malbum', '-')),
						'songname': filterBadCharacter(response_json.get('msong', '-')),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(response_json.get('msong', '-'))]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': seconds2hms(duration)
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''初始化'''
	def __initialize(self):
		self.headers = {
						'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_5) AppleWebKit/605.1.15 (KHTML, like Gecko)',
						'Cookie': 'wmid=142420656; user_type=1; country=id; session_key=2a5d97d05dc8fe238150184eaf3519ad;',
						'X-Forwarded-For': '36.73.34.109'
					}
		self.search_url = 'https://api-jooxtt.sanook.com/web-fcgi-bin/web_search'
		self.songinfo_url = 'https://api.joox.com/web-fcgi-bin/web_get_songinfo'


'''
Function:
	一些终端打印工具
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import logging
from prettytable import PrettyTable


'''打印日志类'''
class Logger():
	def __init__(self, logfilepath, **kwargs):
		logging.basicConfig(level=logging.INFO,
							format='%(asctime)s %(levelname)-8s %(message)s',
							datefmt='%Y-%m-%d %H:%M:%S',
							handlers=[logging.FileHandler(logfilepath),
									  logging.StreamHandler()])
	@staticmethod
	def log(level, message):
		logging.log(level, message)
	@staticmethod
	def debug(message):
		Logger.log(logging.DEBUG, message)
	@staticmethod
	def info(message):
		Logger.log(logging.INFO, message)
	@staticmethod
	def warning(message):
		Logger.log(logging.WARNING, message)
	@staticmethod
	def error(message):
		Logger.log(logging.ERROR, message)


'''打印表格'''
def printTable(title, items):
	assert isinstance(title, list) and isinstance(items, list), '<title> and <items> should be list in printTable...'
	table = PrettyTable(title)
	for item in items: table.add_row(item)
	print(table)
	return table


'''
Function:
	咪咕音乐下载: http://www.migu.cn/
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import requests
from ..utils.misc import *
from ..utils.downloader import Downloader


'''咪咕音乐下载类'''
class migu():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'migu'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		params = {
					'ua': 'Android_migu',
					'version': '5.0.1',
					'text': keyword,
					'pageNo': '1',
					'pageSize': cfg['search_size_per_source'],
					'searchSwitch': '{"song":1,"album":0,"singer":0,"tagSong":0,"mvSong":0,"songlist":0,"bestShow":1}',
				}
		response = self.session.get(self.search_url, headers=self.headers, params=params)
		all_items = response.json()['songResultData']['result']
		songinfos = []
		for item in all_items:
			ext = ''
			download_url = ''
			filesize = '-MB'
			for rate in sorted(item.get('rateFormats', []), key=lambda x: int(x['size']), reverse=True):
				if (int(rate['size']) == 0) or (not rate.get('formatType', '')) or (not rate.get('resourceType', '')): continue
				ext = 'flac' if rate.get('formatType') == 'SQ' else 'mp3'
				download_url = self.player_url.format(copyrightId=item['copyrightId'], 
													  contentId=item['contentId'], 
													  toneFlag=rate['formatType'],
													  resourceType=rate['resourceType'])
				filesize = str(round(int(rate['size'])/1024/1024, 2)) + 'MB'
				break
			if not download_url: continue
			duration = '-:-:-'
			songinfo = {
						'source': self.source,
						'songid': str(item['id']),
						'singers': filterBadCharacter(','.join([s.get('name', '') for s in item.get('singers', [])])),
						'album': filterBadCharacter(item.get('albums', [{'name': '-'}])[0].get('name', '-')),
						'songname': filterBadCharacter(item.get('name', '-')),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(item.get('name', '-'))]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': duration
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''初始化'''
	def __initialize(self):
		self.headers = {
						'Referer': 'https://m.music.migu.cn/', 
						'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.75 Mobile Safari/537.36'
					}
		self.search_url = 'http://pd.musicapp.migu.cn/MIGUM3.0/v1.0/content/search_all.do'
		self.player_url = 'https://app.pd.nf.migu.cn/MIGUM3.0/v1.0/content/sub/listenSong.do?channel=mx&copyrightId={copyrightId}&contentId={contentId}&toneFlag={toneFlag}&resourceType={resourceType}&userId=15548614588710179085069&netType=00'


'''
Function:
	虾米音乐下载: https://www.xiami.com/
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import re
import time
import json
import requests
from hashlib import md5
from ..utils.misc import *
from ..utils.downloader import Downloader


'''虾米音乐下载类'''
class xiami():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'xiami'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		token = self.__getToken()
		search_url = self.base_url.format(action=self.actions['searchsongs'])
		params = {
					'key': keyword,
					'pagingVO': {'page': '1', 'pageSize': str(cfg['search_size_per_source'])}
				}
		response = self.session.get(search_url, headers=self.headers, params=self.__xiamiSign(params, token))
		all_items = response.json()['data']['data']['songs']
		songinfos = []
		for item in all_items:
			download_url = ''
			for file in item['listenFiles']:
				if not file['downloadFileSize']: continue
				filesize = str(round(int(file['downloadFileSize'])/1024/1024, 2)) + 'MB'
				download_url = file['listenFile']
				ext = file['format']
				duration = int(file.get('length', 0)) / 1000
				break
			if not download_url: continue
			songinfo = {
						'source': self.source,
						'songid': str(item['songId']),
						'singers': filterBadCharacter(item.get('artistName', '-')),
						'album': filterBadCharacter(item.get('albumName', '-')),
						'songname': filterBadCharacter(item.get('songName', '-')).split('–')[0].strip(),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(item.get('songName', '-')).split('–')[0].strip()]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': seconds2hms(duration)
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''虾米签名'''
	def __xiamiSign(self, params, token=''):
		appkey = '23649156'
		t = str(int(time.time() * 1000))
		request_str = {
						'header': {'appId': '200', 'platformId': 'h5'},
						'model': params
					}
		data = json.dumps({'requestStr': json.dumps(request_str)})
		sign = '%s&%s&%s&%s' % (token, t, appkey, data)
		sign = md5(sign.encode('utf-8')).hexdigest()
		params = {
					't': t,
					'appKey': appkey,
					'sign': sign,
					'data': data
				}
		return params
	'''获得请求所需的token'''
	def __getToken(self):
		action = self.actions['getsongdetail']
		url = self.base_url.format(action=action)
		params = {'songId': '1'}
		response = self.session.get(url, params=self.__xiamiSign(params))
		cookies = response.cookies.get_dict()
		return cookies['_m_h5_tk'].split('_')[0]
	'''初始化'''
	def __initialize(self):
		self.headers = {
						'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36',
						'Referer': 'http://h.xiami.com',
						'Connection': 'keep-alive',
						'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
						'Accept-Encoding': 'gzip,deflate,sdch',
						'Accept': '*/*'
					}
		self.base_url = 'https://acs.m.xiami.com/h5/{action}/1.0/'
		self.actions = {
						'searchsongs': 'mtop.alimusic.search.searchservice.searchsongs',
						'getsongdetail': 'mtop.alimusic.music.songservice.getsongdetail',
						'getsongs': 'mtop.alimusic.music.songservice.getsongs',
						'getsonglyrics': 'mtop.alimusic.music.lyricservice.getsonglyrics'
					}


'''
Function:
	网易云音乐下载: https://music.163.com/
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import os
import base64
import codecs
import requests
from ..utils.misc import *
from Crypto.Cipher import AES
from ..utils.downloader import Downloader


'''
Function:
	用于算post的两个参数, 具体原理详见知乎：
	https://www.zhihu.com/question/36081767
'''
class Cracker():
	def __init__(self):
		self.modulus = '00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b3ece0462db0a22b8e7'
		self.nonce = '0CoJUm6Qyw8W8jud'
		self.pubKey = '010001'
	def get(self, text):
		text = json.dumps(text)
		secKey = self._createSecretKey(16)
		encText = self._aesEncrypt(self._aesEncrypt(text, self.nonce), secKey)
		encSecKey = self._rsaEncrypt(secKey, self.pubKey, self.modulus)
		post_data = {
					'params': encText,
					'encSecKey': encSecKey
					}
		return post_data
	def _aesEncrypt(self, text, secKey):
		pad = 16 - len(text) % 16
		if isinstance(text, bytes):
			text = text.decode('utf-8')
		text = text + str(pad * chr(pad))
		secKey = secKey.encode('utf-8')
		encryptor = AES.new(secKey, 2, b'0102030405060708')
		text = text.encode('utf-8')
		ciphertext = encryptor.encrypt(text)
		ciphertext = base64.b64encode(ciphertext)
		return ciphertext
	def _rsaEncrypt(self, text, pubKey, modulus):
		text = text[::-1]
		rs = int(codecs.encode(text.encode('utf-8'), 'hex_codec'), 16) ** int(pubKey, 16) % int(modulus, 16)
		return format(rs, 'x').zfill(256)
	def _createSecretKey(self, size):
		return (''.join(map(lambda xx: (hex(ord(xx))[2:]), str(os.urandom(size)))))[0:16]


'''网易云音乐下载类'''
class netease():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'netease'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.cracker = Cracker()
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		params = {
					's': keyword,
					'type': '1',
					'offset': '0',
					'sub': 'false',
					'limit': cfg['search_size_per_source']
				}
		response = self.session.post(self.search_url, headers=self.headers, params=params, data=self.cracker.get(params))
		all_items = response.json()['result']['songs']
		songinfos = []
		for item in all_items:
			if item['privilege']['fl'] == 0: continue
			for q in ['h', 'm', 'l']:
				params = {
							'ids': [item['id']],
							'br': item[q]['br'],
							'csrf_token': ''
						}
				response = self.session.post(self.player_url, headers=self.headers, data=self.cracker.get(params))
				response_json = response.json()
				if response_json.get('code') == 200: break
			if response_json.get('code') != 200: continue
			download_url = response_json['data'][0]['url']
			if not download_url: continue
			filesize = str(round(int(item[q]['size'])/1024/1024, 2)) + 'MB'
			ext = download_url.split('.')[-1]
			duration = int(item.get('dt', 0) / 1000)
			songinfo = {
						'source': self.source,
						'songid': str(item['id']),
						'singers': filterBadCharacter(','.join([s.get('name', '') for s in item.get('ar')])),
						'album': filterBadCharacter(item.get('al', {}).get('name', '-')),
						'songname': filterBadCharacter(item.get('name', '-')),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(item.get('name', '-'))]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': seconds2hms(duration)
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''初始化'''
	def __initialize(self):
		self.headers = {
						'Accept': '*/*',
						'Accept-Encoding': 'gzip,deflate,sdch',
						'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
						'Connection': 'keep-alive',
						'Content-Type': 'application/x-www-form-urlencoded',
						'Host': 'music.163.com',
						'Origin': 'https://music.163.com',
						'Referer': 'https://music.163.com/',
						'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.32 Safari/537.36'
					}
		self.search_url = 'http://music.163.com/weapi/cloudsearch/get/web?csrf_token='
		self.player_url = 'http://music.163.com/weapi/song/enhance/player/url?csrf_token='


'''
Function:
	酷我音乐下载: http://www.kuwo.cn/
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import time
import requests
from ..utils.misc import *
from ..utils.downloader import Downloader


'''酷我音乐下载类'''
class kuwo():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'kuwo'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		params = {
					'key': keyword,
					'pn': '1',
					'rn': cfg['search_size_per_source'],
					'reqId': 'ffa3dc80-73c2-11ea-a715-7de8a8cc7b68'
				}
		response = self.session.get(self.search_url, headers=self.headers, params=params)
		all_items = response.json()['data']['list']
		songinfos = []
		for item in all_items:
			params = {
						'format': 'mp3',
						'rid': str(item['rid']),
						'response': 'url',
						'type': 'convert_url3',
						'br': '128kmp3',
						'from': 'web',
						't': str(int(time.time()*1000)),
						'reqId': 'de97aac1-73c3-11ea-a715-7de8a8cc7b68'
					}
			response = self.session.get(self.player_url, headers=self.headers, params=params)
			response_json = response.json()
			if response_json.get('code') != 200: continue
			download_url = response_json['url']
			if not download_url: continue
			filesize = '-MB'
			ext = download_url.split('.')[-1]
			duration = int(item.get('duration', 0))
			songinfo = {
						'source': self.source,
						'songid': str(item['rid']),
						'singers': filterBadCharacter(item.get('artist', '-')),
						'album': filterBadCharacter(item.get('album', '-')),
						'songname': filterBadCharacter(item.get('name', '-')),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(item.get('name', '-'))]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': seconds2hms(duration)
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''初始化'''
	def __initialize(self):
		self.headers = {
						'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36',
						'csrf': '0HQ0UGKNAKR',
						'Host': 'www.kuwo.cn',
						'Referer': 'http://www.kuwo.cn/search/list',
						'Cookie': 'kw_token=0HQ0UGKNAKR;'
					}
		self.search_url = 'http://www.kuwo.cn/api/www/search/searchMusicBykeyWord'
		self.player_url = 'http://www.kuwo.cn/url'


'''
Function:
	下载器类
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import click
import warnings
import requests
from .misc import *
from contextlib import closing
warnings.filterwarnings('ignore')


'''下载器类'''
class Downloader():
	def __init__(self, songinfo, session=None, **kwargs):
		self.songinfo = songinfo
		self.session = requests.Session() if session is None else session
		self.__initialize(songinfo['source'])
	'''外部调用'''
	def start(self):
		songinfo, session, headers = self.songinfo, self.session, self.headers
		checkDir(songinfo['savedir'])
		try:
			is_success = False
			with closing(session.get(songinfo['download_url'], headers=headers, stream=True, verify=False)) as response:
				total_size = int(response.headers['content-length'])
				chunk_size = 1024
				if response.status_code == 200:
					label = '[FileSize]: %0.2fMB' % (total_size/1024/1024)
					with click.progressbar(length=total_size, label=label) as progressbar:
						with open(os.path.join(songinfo['savedir'], songinfo['savename']+'.'+songinfo['ext']), "wb") as fp:
							for chunk in response.iter_content(chunk_size=chunk_size):
								if chunk:
									fp.write(chunk)
									progressbar.update(chunk_size)
					is_success = True
		except:
			is_success = False
		return is_success
	'''初始化'''
	def __initialize(self, source):
		if source == 'baiduFlac':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36',
							'Referer': 'http://music.baidu.com/'
						}
		elif source == 'kugou':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
							'Host': 'webfs.yun.kugou.com',
							'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
							'Accept-Encoding': 'gzip, deflate, br',
							'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
							'Cache-Control': 'max-age=0',
							'Connection': 'keep-alive'
						}
		elif source == 'kuwo':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36'
						}
		elif source == 'netease':
			self.headers = {}
		elif source == 'qianqian':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36',
							'Referer': 'http://music.baidu.com/'
						}
		elif source == 'qq':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
							'Referer': 'http://y.qq.com'
						}
		elif source == 'migu':
			self.headers = {
							'Referer': 'https://m.music.migu.cn/', 
							'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.75 Mobile Safari/537.36'
						}
		elif source == 'xiami':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36'
						}
		elif source == 'joox':
			self.headers = {
							'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_5) AppleWebKit/605.1.15 (KHTML, like Gecko)'
						}
		else:
			raise ValueError('Unsupport download music from source <%s>...' % source)


'''
Function:
	setup
Author:
	Charles
微信公众号:
	Charles的皮卡丘
GitHub:
	https://github.com/CharlesPikachu
'''
import musicdl
from setuptools import setup, find_packages


'''readme'''
with open('README.md', 'r', encoding='utf-8') as f:
	long_description = f.read()


'''setup'''
setup(
	name='musicdl',
	version=musicdl.__version__,
	description='Music Downloader.',
	long_description=long_description,
	long_description_content_type='text/markdown',
	classifiers=[
			'License :: OSI Approved :: MIT License',
			'Programming Language :: Python :: 3',
			'Intended Audience :: Developers',
			'Operating System :: OS Independent'],
	author='Charles',
	url='https://github.com/CharlesPikachu/Music-Downloader',
	author_email='charlesblwx@gmail.com',
	license='MIT',
	include_package_data=True,
	install_requires=['requests >= 2.22.0', 'pycryptodome >= 3.8.1', 'click >= 7.0', 'prettytable >= 0.7.2'],
	zip_safe=True,
	packages=find_packages()
)


'''
Function:
	酷狗音乐下载: http://www.kugou.com/
Author:
	Charles
微信公众号:
	Charles的皮卡丘
'''
import time
import requests
from ..utils.misc import *
from ..utils.downloader import Downloader


'''酷狗音乐下载类'''
class kugou():
	def __init__(self, config, logger_handle, **kwargs):
		self.source = 'kugou'
		self.session = requests.Session()
		self.session.proxies.update(config['proxies'])
		self.config = config
		self.logger_handle = logger_handle
		self.__initialize()
	'''歌曲搜索'''
	def search(self, keyword):
		self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
		cfg = self.config.copy()
		params = {
					'keyword': keyword,
					'page': '1',
					'pagesize': cfg['search_size_per_source'],
					'userid': '-1',
					'clientver': '',
					'platform': 'WebFilter',
					'tag': 'em',
					'filter': '',
					'iscorrection': '1',
					'privilege_filter': '0',
					'_': str(int(time.time() * 1000))
				}
		response = self.session.get(self.search_url, headers=self.search_headers, params=params)
		all_items = response.json()['data']['lists']
		songinfos = []
		for item in all_items:
			params = {
						'r': 'play/getdata',
						'hash': str(item['FileHash']),
						'album_id': str(item['AlbumID']),
						'dfid': '1aAcF31Utj2l0ZzFPO0Yjss0',
						'mid': 'ccbb9592c3177be2f3977ff292e0f145',
						'platid': '4',
						'_': str(int(time.time() * 1000))
					}
			response = self.session.get(self.hash_url, headers=self.hash_headers, params=params)
			response_json = response.json()
			if response_json.get('err_code') != 0: continue
			download_url = response_json['data']['play_url'].replace('\\', '')
			if not download_url: continue
			filesize = str(round(int(response_json['data']['filesize'])/1024/1024, 2)) + 'MB'
			ext = download_url.split('.')[-1]
			duration = int(item.get('Duration', 0))
			songinfo = {
						'source': self.source,
						'songid': str(item['ID']),
						'singers': filterBadCharacter(item.get('SingerName', '-')),
						'album': filterBadCharacter(item.get('AlbumName', '-')),
						'songname': filterBadCharacter(item.get('SongName', '-')),
						'savedir': cfg['savedir'],
						'savename': '_'.join([self.source, filterBadCharacter(item.get('SongName', '-'))]),
						'download_url': download_url,
						'filesize': filesize,
						'ext': ext,
						'duration': seconds2hms(duration)
					}
			songinfos.append(songinfo)
		return songinfos
	'''歌曲下载'''
	def download(self, songinfos):
		for songinfo in songinfos:
			self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
			task = Downloader(songinfo, self.session)
			if task.start():
				self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
			else:
				self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
	'''初始化'''
	def __initialize(self):
		self.search_headers = {
								'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
								'Referer': 'https://www.kugou.com/yy/html/search.html'
							}
		self.hash_headers = {
								'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
								'Referer':'https://www.kugou.com/song/'
							}
		self.search_url = 'http://songsearch.kugou.com/song_search_v2'
		self.hash_url = 'https://wwwapi.kugou.com/yy/index.php'